@@ -1,7 +1,7 @@
 package Pcore::Util::URI::Host;
 
 use Pcore -class;
-use Pcore::Util::Text qw[encode_utf8];
+use Pcore::Util::Text qw[decode_utf8 encode_utf8];
 use AnyEvent::Socket qw[];
 use Pcore::Util::URI::Punycode qw[:ALL];
 
@@ -40,6 +40,16 @@ our $PUB_SUFFIX;
 
 # NOTE host should be in UTF-8 or ASCII punycoded, UTF-8 encoded - is invalid value
 around new => sub ( $orig, $self, $host ) {
+
+    # removing double "."
+    $host =~ s/[.]+/./smg if index( $host, q[..] ) != -1;
+
+    # removing leading "."
+    substr $host, 0, 1, q[] if substr( $host, 0, 1 ) eq q[.];
+
+    # removing trailing "."
+    substr $host, -1, 1, q[] if substr( $host, -1, 1 ) eq q[.];
+
     $host = domain_to_ascii($host) if utf8::is_utf8($host);
 
     return bless { name => lc $host }, __PACKAGE__;
@@ -48,38 +58,45 @@ around new => sub ( $orig, $self, $host ) {
 sub update_all ( $self ) {
 
     # update TLD
-    say 'updating tld.dat';
+    print 'updating tld.dat ... ';
 
     if ( my $res = P->http->get( 'https://data.iana.org/TLD/tlds-alpha-by-domain.txt', buf_size => 0, on_progress => 0 ) ) {
         $ENV->share->store( '/data/tld.dat', \encode_utf8( join $LF, sort map { domain_to_utf8(lc) } grep { $_ && !/\A\s*#/sm } split /\n/sm, $res->body->$* ), 'Pcore' );
 
         undef $TLD;
+
+        say 'done';
     }
     else {
+        say 'error';
+
         return 0;
     }
 
     # update pub. suffixes, should be updated after TLDs
-    say 'updating pub_suffix.dat';
+    print 'updating pub_suffix.dat ... ';
 
     if ( my $res = P->http->get( 'https://publicsuffix.org/list/effective_tld_names.dat', buf_size => 0, on_progress => 0 ) ) {
         my $suffixes = {};
 
+        decode_utf8 $res->body->$*;
+
         for my $domain ( split /\n/sm, $res->body->$* ) {
 
-            # ignore comments
-            next if $domain =~ m[//]sm;
+            # remove spaces
+            $domain =~ s/\s//smg;
+
+            # remove comments
+            $domain =~ s[//.*][]sm;
 
             # ignore empty lines
-            next if $domain =~ /\A\s*\z/sm;
+            next if $domain eq q[];
 
             $suffixes->{$domain} = 1;
         }
 
         # add tlds
         for my $tld ( keys $self->tlds->%* ) {
-            utf8::encode($tld);
-
             $suffixes->{$tld} = 1;
         }
 
@@ -98,11 +115,15 @@ sub update_all ( $self ) {
             }
         }
 
-        $ENV->share->store( '/data/pub_suffix.dat', \join( $LF, sort keys $suffixes->%* ), 'Pcore' );
+        $ENV->share->store( '/data/pub_suffix.dat', \join( $LF, sort map { encode_utf8 $_} keys $suffixes->%* ), 'Pcore' );
 
         undef $PUB_SUFFIX;
+
+        say 'done';
     }
     else {
+        say 'error';
+
         return 0;
     }
 
@@ -116,7 +137,13 @@ sub tlds ( $self ) {
 }
 
 sub pub_suffixes ( $self ) {
-    $PUB_SUFFIX = { map { $_ => 1 } P->file->read_lines( $ENV->share->get('/data/pub_suffix.dat') )->@* } if !$PUB_SUFFIX;
+    if ( !$PUB_SUFFIX ) {
+        my $buf = P->file->read_bin( $ENV->share->get('/data/pub_suffix.dat') );
+
+        decode_utf8 $buf->$*;
+
+        $PUB_SUFFIX = { map { $_ => 1 } grep { defined && $_ ne q[] } split /\n/sm, $buf->$* };
+    }
 
     return $PUB_SUFFIX;
 }
@@ -292,16 +319,20 @@ sub _build_pub_suffix_utf8 ($self) {
 sub _build_is_root_domain ($self) {
     return 0 unless $self->is_domain;
 
-    return $self->root_domain eq $self->name ? 1 : 0;
+    return length( $self->root_domain ) eq length( $self->name ) ? 1 : 0;
 }
 
 sub _build_root_domain ($self) {
     return q[] unless $self->is_domain;
 
     if ( my $pub_suffix = $self->pub_suffix ) {
-        if ( $self->canon =~ /\A.*?([^.]+[.]$pub_suffix)\z/sm ) {
-            return $1;
-        }
+        my $canon = $self->canon;
+
+        return q[] if length $pub_suffix >= length $canon;
+
+        my $root = substr $canon, 0, length($canon) - length($pub_suffix) - 1;
+
+        return ( split /[.]/sm, $root )[-1] . ".$pub_suffix";
     }
 
     return q[];
@@ -318,9 +349,9 @@ sub _build_root_domain_utf8 ($self) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 80, 87, 101          | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
+## |    3 | 99, 104, 118         | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 261, 264             | ControlStructures::ProhibitDeepNests - Code structure is deeply nested                                         |
+## |    3 | 288, 291             | ControlStructures::ProhibitDeepNests - Code structure is deeply nested                                         |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
